#include "TableCsv.h"

bool TableCsv::openSaveClose(std::string & fileName) {
    using namespace std;

// Trimming
    string s = buffer->str();
    int size = s.length();
    string c;
    for(int i = 0; i < size; i += NEW_LINE_LENGTH) {
        c = s.substr(i, NEW_LINE_LENGTH);
        if(NEW_LINE == c) {
            continue;
        } else {
            s = s.substr(i);
            break;
        }

    }

// Writing
    ofstream saveFile( fileName.c_str() );
    if( saveFile.fail() ) {
        error = "e7: " + fileName + CLOSE_BRACKET;
        return false;
    }

    saveFile << s;
    saveFile.close();
    return true;
}

bool TableCsv::sorting(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e26: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[TO_BUFFER_SIZE];
    priority_queue<string> pq;

    string *p = new string(c);
    for(int line = 1; line <= rowCount; line += 1 ) {
        if( in.getline(c, BUFFER_SIZE) ) {
            delete p;
            p = new string(c);
            pq.push(*p);
        } else {
            break;
        }

    }

    stringstream ss;
    while(! pq.empty() ) {
		ss << pq.top() << NEW_LINE;
        pq.pop();
    }

    string s = ss.str();
    if(whoText) {
        textName = WHO_TEXT;
    } else {
        TimeName wt(WHO, CSV_FILE);
        textName = wt.temporaryName(serialNumber);
        cout << textName << endl;
    }

// Writing
    ofstream file( textName.c_str() );
    file << s;
    file.close();

    delete p;
    return true;
}

int TableCsv::countBalls(std::string & s) {
    using namespace std;

    int count = 0;
    int q;

    q = s.find(draw[1]);
    if(q >= 0) {
        count += 1;
    }

    q = s.find(draw[2]);
    if(q >= 0) {
        count += 1;
    }

    q = s.find(draw[3]);
    if(q >= 0) {
        count += 1;
    }

    q = s.find(draw[4]);
    if(q >= 0) {
        count += 1;
    }

    q = s.find(draw[5]);
    if(q >= 0) {
        count += 1;
    }

    q = s.find(draw[6]);
    if(q >= 0) {
        count += 1;
    }

    q = s.find(draw[7]);
    if(q >= 0) {
        count += 1;
    }

    return count;
}

std::string TableCsv::assembling() {
    using namespace std;

    stringstream *os = new stringstream();

    for(int i = 1; i < DRAW_ARRAY_SIZE; ++i) {
        if( a[i].length() > 0 ) {
            *os << a[i] << GAP;
        }

    }

    string s = os->str();
    return s;
}

void TableCsv::dividing(char *p, char *to, int to_array_size) {
    using namespace std;

    char *from;
    int n, u;
    string *to_string = new string("");

    a[0] = EMPTY;
    memset(to, 0, to_array_size);
    for(int i = 1; i < DRAW_ARRAY_SIZE; ++i) {
        n = strspn(p, "0123456789");
        from = p;
        p += n;

        u = strcspn(p, "0123456789");
        p += u;

        memcpy(to, from, n);
        to[n] = '\0';
        delete to_string;
        to_string = new string(to);
        a[i] = *to_string;
    }

    delete to_string;
}

void TableCsv::listing() {
    using namespace std;

    if(split == 0) {
        return;
    }

    switch(count) {
    case 0:
        return;
    default:
        break;
    }

    int q;
    string head;
    string tail = combos->str();
    for(;;) {
        q = tail.find(SEPARATOR);
        if(q != string::npos) {
            head = tail.substr(0, q);
            q += SEPARATOR_SIZE;
            tail = tail.substr(q);

            if(scope) {
                *buffer << head << COMMA << ALL_QP << whose << NEW_LINE;
            } else {
                *buffer << head << COMMA << EMPTY << whose << NEW_LINE;
            }

            total += 1;
            q = strcspn( tail.c_str(), "0123456789");
            tail = tail.substr(q);
        } else {
            break;
        }

    }

}

TableCsv::TableCsv(int r, bool s, bool w): ALL_QP("ALL QP"), ALL_QP_SIZE(6), CLOSE_BRACKET(" )"), COMMA(","), EMPTY(""), GAP(" "), NEW_LINE("\n"), NEW_LINE_LENGTH(1), QP("QP"), SEPARATOR("`"), SEPARATOR_SIZE(1), SORT_IN_REVERSE_ORDER(true) { // Shift_JIS
    buffer = new std::stringstream();
    combos = NULL;
    rowCount = r;
    serialNumber = s;
    total = 0;
    whoText = w;

    std::string empty(EMPTY);
    clear(empty, 0); // combos = new std::stringstream();
}

bool TableCsv::run(std::string & fileName) {
    using namespace std;

    ifstream in( fileName.c_str() );
    if( in.fail() ) {
        error = "e22: ifstream in( " + fileName + CLOSE_BRACKET;
        return false;
    }

    char c[TO_BUFFER_SIZE];
    char to[TO_BUFFER_SIZE];
    if( in.getline(c, TO_BUFFER_SIZE) ) {
// Line-1
        if( strlen(c) > STANDARD_LINE_LENGTH ) {
            dividing(c, to, TO_BUFFER_SIZE);
            for(int i = 0; i < DRAW_ARRAY_SIZE; ++i) {
                draw[i] = a[i];
            }

        } else {
            const string LINE_LENGTH_ERROR("e10(Line-1): strlen(c) <= STANDARD_LINE_LENGTH");
            error = LINE_LENGTH_ERROR;
            return false;
        }

    } else {
        const string READ_ERROR("e11(Line-1): in.getline(c, TO_BUFFER_SIZE) ");
        error = READ_ERROR;
        return false;
    }

    int b, howMany, r;
    string *p = new string("");
    string s;
    Characterizable *lineType = new DateFollowed(*p);

    if(whoText) {
        const string who_text(WHO_TEXT);
        textName = who_text;
    } else {
        TimeName wt(WHO, CSV_FILE);
        textName = wt.temporaryName(serialNumber);
        cout << textName << endl;
    }

    while( in.getline(c, BUFFER_SIZE) ) {
        scope = allQP(c);

        if( strlen(c) > STANDARD_LINE_LENGTH ) {
            delete p;
            p = new string(c);
            s = *p;

            delete lineType;
            lineType = new DateFollowed(s);
            r = lineType->dateLine();
            if(r > 0) {
                listing();
                s = lineType->common();
                clear(s, r);
                continue;
            }

            dividing(c, to, TO_BUFFER_SIZE);
        } else {
            continue;
        }

        howMany = countBalls(s);
        if(howMany > 0) {
            s = assembling();
            add(howMany, s);
        }

    }

    in.close();
    listing();
    openSaveClose(textName);
    sorting(textName);

    delete lineType;
    delete p;
    return true;
}
